/*

[jquery tunable radiobox]
ver    :  1.0.4
author :  Andriy Sokolovskiy (sokandpal@yandex.ru)

*/

(function($) {
  
  
  var methods = {
    init: function (userOptions, userText) {
    
      //default settings 
      var settings = $.extend( {
          type       : undefined,
          count      : 0,
          width      : 12,
          height     : 12,
          time       : 5,
          fillColor  : '#000000',
          checked    : null
      }, userOptions);
      
      userTextTemp = userText;
      
      
      if(settings.count == 0)
          $.error('must have 1 and more elements');      
      if(settings.type === undefined)
          $.error('must have an initialized button type');
      if(typeof userText != 'string')
          $.error('userText must be a string');
      
      if(settings.type == 'checkbox')
          methods.checkbox(settings);
      if(settings.type == 'radio')
          methods.radio(settings);    
      
    },
    
    checkbox : function(settings) {
        
        /**/
        var centerX = (settings.width/Math.sqrt(2))/3;
        var centerY = (settings.height/Math.sqrt(2))/3;
        var side    = settings.width*0.55;
        
        var tempSide = 0;
		    var userTextArr = userTextTemp.split(' ');
        var checkedlist = [];
        /**/
        
        // some CSS stuff
        for(var i = 0; i < settings.count; i++) {
            that.append('<div class="button-container"><canvas class="checkbox-element"></canvas><div class="button-text">'+userTextArr[i]+'</div></div>');
            that.find('.button-text').css('line-height', settings.height+'px');
            that.find('.checkbox-element').attr('width', settings.width).attr('height', settings.height);
        }
        that.append('<div class="checked-elements" style="display: none;"></div>');
        var targetChecked = that.find('.checked-elements');
        
        
        // drawing function
        
        var draw = function(ctx, elem) {
             var check = $(elem).is('[checked]');
             var checkedText = $(elem).parent().find('.button-text').text();
             
             
             if(check) {
                 // here we got nice canvas clearing:
                 
                 ctx.clearRect(0, 0, settings.width, settings.height);
                 $(elem).removeAttr('checked');
                 
                 /* save checking elements */
                 if(checkedlist.indexOf(checkedText) > -1) {
                      checkedlist.remove(checkedText);
                 }
                 checkedlistTemp = checkedlist.join(' ');
                 targetChecked.attr('checked-list', checkedlistTemp);               
                 /* */
                 
             }
             else {
             
                // fadein effect of cheking
             
             var fade = setInterval(function() {
                  ctx.fillStyle = settings.fillColor;
                  ctx.fillRect(centerX, centerY, tempSide, tempSide);
                  
                  if(tempSide >= side) { 
                      clearInterval(fade); 
                      tempSide = 0;
                      $(elem).attr('checked', '1');
                      
                      /* save checking elements */                      
                      if(!(checkedlist.indexOf(checkedText) > -1)) {
                        checkedlist.push(checkedText);
                      }
                      checkedlistTemp = checkedlist.join(' ');
                      targetChecked.attr('checked-list', checkedlistTemp);
                      /* */
                      
                      return;
                  }
                  tempSide+=0.5;
                   
                  }, settings.time);
             
             }
            
        };
        
        //preparing canvas
        
        var checkCanvas = that.find('.checkbox-element');
        var ctxCanvas = [];
        checkCanvas.each(function(i, elem) {
            ctxCanvas[i] = elem.getContext('2d');
			
            $(elem).click(function() {
                draw(ctxCanvas[i], elem);
            });      
            
            // check elements from prechecked-list
         
            if(settings.checked != null) {
              l = settings.checked.length;
              while(l--) {
                  if(settings.checked[l] == i)
                  draw(ctxCanvas[i], elem);
              }
            }   
        });
       
    },
    
    radio : function(settings) {
        /**/
        var centerX = settings.width / 2;
        var centerY = settings.height / 2;
        var radius  = settings.width*0.3;
        
        var tempRad = 0; 
        var userTextArr = userTextTemp.split(' ');
        var checkedlist = [];
        /**/
        
        // some CSS stuff
        for(var i = 0; i < settings.count; i++) {
            that.append('<div class="button-container"><canvas class="radio-element"></canvas><div class="button-text">'+userTextArr[i]+'</div></div>');
            that.find('.button-text').css('line-height', settings.height+'px');
            that.find('.radio-element').attr('width', settings.width).attr('height', settings.height);
        }
        that.append('<div class="checked-elements" style="display: none;"></div>');
        var targetChecked = that.find('.checked-elements');  
        
        
        var draw = function(ctx, elem) {
            var check = $(elem).is('[checked]');
            var checkedText = $(elem).parent().find('.button-text').text();
            
            if(check) return;
            else {
                // fadein effect of cheking
                
                var fade = setInterval(function() {
            
                    $(checkCanvas).each(function(i, element) {
                        $(element).removeAttr('checked');
                        element.getContext('2d').clearRect(0, 0, settings.width, settings.height);
                        element.width = element.width;                    
                    });
                        
                    ctx.arc(centerX, centerY, tempRad, 0 , 2 * Math.PI, false);
                    ctx.fillStyle = settings.fillColor;
                    ctx.fill();
                        
                    if(tempRad >= radius) { 
                        clearInterval(fade); 
                        tempRad = 0;
                        if(!check) {
                          $(elem).attr('checked', '1');
                          
                          /* save checking elements */                      
                          checkedlist = [];
                          checkedlist.push(checkedText);

                          checkedlistTemp = checkedlist.join(' ');
                          targetChecked.attr('checked-list', checkedlistTemp);
                          /* */
                          
                          return;
                        }
                        
                    }
                    tempRad+=0.5;  
                }, settings.time); 
            } 
        };
        
        //preparing canvas
        
        var checkCanvas = that.find('.radio-element');
        var ctxCanvas = [];
        checkCanvas.each(function(i, elem) {
            ctxCanvas[i] = elem.getContext("2d");
            
            $(elem).click(function() {
                draw(ctxCanvas[i], elem);
            });
            
            // pre-cheking radio element
            if(settings.checked != null) {
              if(!isNaN(parseFloat(settings.checked)) && isFinite(settings.checked)) {
                if(settings.checked == i)
                draw(ctxCanvas[i], elem);
              }
            }            
        
        });
        
    },

    getchecked: function() {
        // returning checked elements array
        return (that.find('.checked-elements').attr('checked-list')).split(' ');
    }
  };
    
  $.fn.radiobox = function(method) {
      
      // taking current element for
      // using in methods
      
      that = this;
            
      // method calling logic
      
      if (methods[method]) {
          return methods[method].apply(this, Array.prototype.slice.call(arguments,1));
      } 
      else if(typeof method === 'object' || ! method) {
          return methods.init.apply(this, arguments);
      } 
      else {
          $.error( 'Can not find method ' +  method);
      }   
      
  };
  
})(jQuery);

// method to remove element by value

Array.prototype.remove = function() {
    var what, a = arguments, L = a.length, ax;
    while (L && this.length) {
        what = a[--L];
        while ((ax = this.indexOf(what)) !== -1) {
            this.splice(ax, 1);
        }
    }
    return this;
};